/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SaveRulePackageCommand.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageSystemNameEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.RulePackageSaveReportVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.AbstractRulePackageVO;

/**
 * Save a given rule package
 *
 * @author Julien Nicoulaud
 */
[Event(name="rulePackageModified", type="org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageSystemNameEvent")]
public class SaveRulePackageCommand extends AbstractCommand {

    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * The traited rule package
     */
    private var synchronizedRulePackage:SynchronizedRulePackage = null;

    /**
     * Close the editor if successfully saved ?
     */
    private var closeAfterSuccessfulSave:Boolean = false;


    public function SaveRulePackageCommand() {
    }

// =========================================================================
    // Execution
    // =========================================================================

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    override public function execute(event:CairngormEvent):void {

        // Store the target rule package
        synchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;

        // Dispatch a notification to Kerneos
        debug("Saving \"" + synchronizedRulePackage.rulePackage.systemName + "\"");

        // Get informations about the actions to do at result
        switch ((event as SynchronizedRulePackageEvent).type) {
            case SynchronizedRulePackageEvent.SAVE_AND_CLOSE_SYNCHRONIZED_RULE_PACKAGE:
                closeAfterSuccessfulSave = true;
                break;
            default:
                closeAfterSuccessfulSave = false;
                break;
        }

        // Execute a server-side method thanks to the delegate
        delegate.responder = this;
        delegate.saveRulePackage(synchronizedRulePackage.rulePackage, model.ruleEngineName);
    }


    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    override public function result(event:Object):void {
        // Get the server response
        var serverResponse:RulePackageSaveReportVO =
                (event as ResultEvent).result as RulePackageSaveReportVO;

        // Retrieve failures and notify the view
        var failures:ArrayCollection = serverResponse.failures as ArrayCollection;

        for each (var failure:ArrayCollection in failures) {

            // Build a server side exception object
            var serverSideException:ServerSideException =
                    new ServerSideException(failure[0], failure[1]);

            // If stacktrace provided, add it
            if (failure.length > 2) {
                serverSideException.stackTrace = failure[2];
            }

            // Build an event and dispatch it
            var serverSideExceptionEvent:ServerSideExceptionEvent =
                    new ServerSideExceptionEvent(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION,
                            serverSideException);
            dispatcher.dispatchEvent(serverSideExceptionEvent);
        }

        // Update the model depending on the action wanted
        if (closeAfterSuccessfulSave && failures.length == 0) {

            // Remove the rule package
            model.ruleEditorModel.removeRulePackage(synchronizedRulePackage);

        } else {

            // If the rule package was activated|deactivated
            if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_ACTIVATE)) {

                // Update the state
                synchronizedRulePackage.rulePackage.state = AbstractRulePackageVO.ACTIVATED;

            } else if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_DEACTIVATE)) {

                // Update the state
                synchronizedRulePackage.rulePackage.state = AbstractRulePackageVO.DEACTIVATED;
            }

            // If it was saved
            if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_SAVE)) {

                // Dispatch a notification event
                var changeEvent:RulePackageSystemNameEvent;
                if (synchronizedRulePackage.rulePackage.previousSystemName !== null) {
                    changeEvent = new RulePackageSystemNameEvent(RulePackageSystemNameEvent.EDITOR_RULE_PACKAGE_MODIFIED,
                            synchronizedRulePackage.rulePackage.previousSystemName);
                } else {
                    changeEvent = new RulePackageSystemNameEvent(RulePackageSystemNameEvent.EDITOR_RULE_PACKAGE_MODIFIED,
                            synchronizedRulePackage.rulePackage.systemName);
                }
                dispatcher.dispatchEvent(changeEvent);

                // Update the opened rule package object
                synchronizedRulePackage.rulePackage.previousSystemName = null;

                // Update the state
                synchronizedRulePackage.state = SynchronizedRulePackageState.SYNCHRONIZED;

            }
        }

    }


}
}
